import 'dart:convert';
import 'dart:io';

import 'package:flutter_extended_wordpress/models/user.dart';
import 'package:meta/meta.dart';
import 'package:http/http.dart' as http;

class HttpService {
  String _baseUrl;
  String _tokenPrefix;
  String _token;
  Map<String, String> _headers = {
    'Authorization': '',
    'Accept': 'application/json',
  };

  HttpService({@required String baseUrl, String tokenPrefix = 'JWT'})
      : _baseUrl = baseUrl,
        this._tokenPrefix = tokenPrefix,
        super();

  set token(tokenString) {
    _token = tokenString;
    _headers.addAll({
      'Authorization': '$_tokenPrefix $tokenString',
    });
  }

  String get baseUrl => _baseUrl;

  Future tokenRefresh() async {
    final StringBuffer url = new StringBuffer(_baseUrl + '/auth/token-refresh');
    final response = await http.post(
      url.toString(),
      body: {'token': _token},
    );
    if (response.statusCode >= 200 && response.statusCode < 300) {
      var responseBody = _bodyFromJson(response);
      final JWTResponse res = JWTResponse.fromJson(responseBody);
      this.token = res.token;
      return responseBody;
    } else {
      HttpError err = HttpError.fromJson(json.decode(response.body));
      throw err;
    }
  }

  Future jwtLogin({
    @required String path,
    dynamic body,
    bool withoutHeader = false,
  }) async {
    final StringBuffer url = new StringBuffer(_baseUrl + path);
    final response = await http.post(
      url.toString(),
      body: body,
    );
    if (response.statusCode >= 200 && response.statusCode < 300) {
      var responseBody = _bodyFromJson(response);
      final JWTResponse res = JWTResponse.fromJson(responseBody);
      this.token = res.token;
      return responseBody;
    } else {
      HttpError err = HttpError.fromJson(json.decode(response.body));
      throw err;
    }
  }

  Future<bool> create({
    @required String path,
    Map<String, String> body,
    List<File> attachments,
  }) async {
    final StringBuffer url = new StringBuffer(_baseUrl + path);
    final http.Response response = await http.post(
      url.toString(),
      body: body,
      headers: _headers,
    );
    if (response.statusCode >= 200 && response.statusCode < 300) {
      return true;
    } else {
      HttpError err = HttpError.fromJson(json.decode(response.body));
      throw err;
    }
  }

  Future<dynamic> read({
    @required String path,
    Map<String, dynamic> params,
  }) async {
    final StringBuffer url = new StringBuffer(_baseUrl + path);
    if (params != null) {
      url.write(_constructUrlParams(params));
    }
    final response = await http.get(
      url.toString(),
      headers: _headers,
    );
    if (response.statusCode >= 200 && response.statusCode < 300) {
      return _bodyFromJson(response);
    } else {
      HttpError err = HttpError.fromJson(json.decode(response.body));
      throw err;
    }
  }

  Future<bool> update({@required String path, dynamic body}) async {
    final StringBuffer url = new StringBuffer(_baseUrl + path);
    final response = await http.put(
      url.toString(),
      body: body,
      headers: _headers,
    );
    if (response.statusCode >= 200 && response.statusCode < 300) {
      return true;
    } else {
      HttpError err = HttpError.fromJson(json.decode(response.body));
      throw err;
    }
  }

  Future<dynamic> delete({@required String path}) async {
    final StringBuffer url = new StringBuffer(_baseUrl + path);
    final response = await http.delete(
      url.toString(),
      headers: _headers,
    );
    if (response.statusCode >= 200 && response.statusCode < 300) {
      return true;
    } else {
      HttpError err = HttpError.fromJson(json.decode(response.body));
      throw err;
    }
  }

  /// Formats a [Map] of parameters to a string of URL friendly parameters.
  String _constructUrlParams(Map<String, dynamic> params) {
    StringBuffer p = new StringBuffer('/?');
    params.forEach((key, value) {
      // if map has multiple values of same key
      if (value is List) {
        value.forEach((v) {
          if (v != null && v != '') {
            p.write('$key=$v');
            p.write('&');
          }
        });
      } else if (value != null && value != '') {
        p.write('$key=$value');
        p.write('&');
      }
    });
    return Uri.encodeFull(p.toString());
  }

  dynamic _bodyFromJson(http.Response response) {
    String body = utf8.decode(response.bodyBytes);
    var parsedData = json.decode(body);
    return parsedData;
  }
}

/// All API related errors are thrown as an object of this class.
class HttpError {
  String code;
  String message;
  Data data;

  HttpError({this.code, this.message, this.data});

  HttpError.fromJson(Map<String, dynamic> json) {
    code = json['code'] != null ? json['code'] : 'unknown_error';
    if (json['message'] != null) {
      message = json['message'];
    } else if (json['detail'] != null) {
      message = json['detail'];
    } else {
      message = 'Unknown error is occurred.';
    }
    data = json['data'] != null ? new Data.fromJson(json['data']) : null;
  }

  Map<String, dynamic> toJson() {
    final Map<String, dynamic> data = new Map<String, dynamic>();
    data['code'] = this.code;
    data['message'] = this.message;
    if (this.data != null) {
      data['data'] = this.data.toJson();
    }
    return data;
  }

  @override
  String toString() {
    return 'Error { code: $code, message: $message, status: ${data ?? data.status} }';
  }
}

class Data {
  int status;

  Data({this.status});

  Data.fromJson(Map<String, dynamic> json) {
    status = json['status'];
  }

  Map<String, dynamic> toJson() {
    final Map<String, dynamic> data = new Map<String, dynamic>();
    data['status'] = this.status;
    return data;
  }
}

class JWTResponse {
  String token;
  User user;

  JWTResponse({this.token, this.user});

  JWTResponse.fromJson(Map<String, dynamic> json) {
    token = json['token'];
    user = json['user'] != null ? User.fromJson(json['user']) : null;
  }

  Map<String, dynamic> toJson() {
    final Map<String, dynamic> data = new Map<String, dynamic>();
    data['token'] = this.token;
    if (this.user != null) data['user'] = this.user.toJson();
    return data;
  }

  @override
  String toString() {
    return 'JWT Response: { user_id: ${user.id}, user_name: ${user.name} }';
  }
}
